/*
**  PROGRAM: Mandelbrot area
**
**  PURPOSE: Program to compute the area of a  Mandelbrot set.
**           Correct answer should be around 1.510659.
**           WARNING: this program may contain errors
**
**  USAGE:   Program runs without input ... just run the executable
**            
**  HISTORY: Written:  (Mark Bull, August 2011).
**           Changed "comples" to "d_comples" to avoid collsion with 
**           math.h complex type (Tim Mattson, September 2011)
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <omp.h>

# define NPOINTS 1000
# define MAXITER 1000


struct d_complex{
   double r;
   double i;
};

void testpoint(struct d_complex, int *dest);

struct d_complex c;
int numoutside = 0;

int image[NPOINTS][NPOINTS] = {0};

int main(){
   int i, j;
   double area, error, eps  = 1.0e-5;


//   Loop over grid of points in the complex plane which contains the Mandelbrot set,
//   testing each point to see whether it is inside or outside the set.

   #pragma omp parallel for default(shared) private(c,/*!!!eps,!!!*//*!!!*/j/*!!!*/)
   for (i=0; i<NPOINTS; i++) {
     for (j=0; j<NPOINTS; j++) {
       c.r = -2.0+2.5*(double)(i)/(double)(NPOINTS)+eps;
       c.i = 1.125*(double)(j)/(double)(NPOINTS)+eps;
       c.i = -0.5 + 2.0*(double)(j)/(double)(NPOINTS)+eps;
       testpoint(/*!!!*/c/*!!!*/, &image[i][j]);
     }
   }

// Calculate area of set and error estimate and output the results
   
   area=2.0*2.5*1.125*(double)(NPOINTS*NPOINTS-numoutside)/(double)(NPOINTS*NPOINTS);
   error=area/(double)NPOINTS;

   printf("Area of Mandlebrot set = %12.8f +/- %12.8f\n",area,error);
   printf("Correct answer should be around 1.510659\n");

   for (i = 0; i < 50; i++) {
     for (j = 0; j < 50; j++) {
       int chunk = NPOINTS / 50 / 2;
       int vals[2][2] = {0};
       for (int ii = 0; ii < 2; ii++) {
         for (int jj = 0; jj < 2; jj++) {
           for (int iii = 0; iii < chunk; iii++) {
             for (int jjj = 0; jjj < chunk; jjj++) {
               vals[ii][jj] += image[i * chunk + ii * chunk + iii][j * chunk + jj * chunk + jjj];
             }
           }
           vals[ii][jj] = vals[ii][jj] < (chunk * chunk) / 2 ? 1 : 0;
         }
       }
       printf("%d", vals[0][0] + vals[0][1] + vals[1][0] + vals[1][1]);
//       if (vals[0][0] && vals[0][1] && vals[1][0] && vals[1][1])
//         printf("\u2588");
//       if (! vals[0][0] && ! vals[0][1] &&   vals[1][0] &&   vals[1][1])
//         printf("\u2580");
//       if (  vals[0][0] &&   vals[0][1] && ! vals[1][0] && ! vals[1][1])
//         printf("\u2584");
//       if (! vals[0][0] &&   vals[0][1] && ! vals[1][0] &&   vals[1][1])
//         printf("\u258C");
//       if (  vals[0][0] && ! vals[0][1] &&   vals[1][0] && ! vals[1][1])
//         printf("\u2590");
//       if (! vals[0][0] && ! vals[0][1] && ! vals[1][0] &&   vals[1][1])
//         printf("\u2597");
//       if (! vals[0][0] && ! vals[0][1] &&   vals[1][0] && ! vals[1][1])
//         printf("\u2596");
//       if (! vals[0][0] &&   vals[0][1] && ! vals[1][0] && ! vals[1][1])
//         printf("\u259D");
//       if (  vals[0][0] && ! vals[0][1] && ! vals[1][0] && ! vals[1][1])
//         printf("\u2598");
//       if (! vals[0][0] &&   vals[0][1] &&   vals[1][0] &&   vals[1][1])
//         printf("\u259F");
//       if (  vals[0][0] && ! vals[0][1] &&   vals[1][0] &&   vals[1][1])
//         printf("\u2599");
//       if (  vals[0][0] &&   vals[0][1] && ! vals[1][0] &&   vals[1][1])
//         printf("\u259C");
//       if (  vals[0][0] &&   vals[0][1] &&   vals[1][0] && ! vals[1][1])
//         printf("\u259B");
//       if (! vals[0][0] &&   vals[0][1] &&   vals[1][0] && ! vals[1][1])
//         printf("\u259A");
//       if (  vals[0][0] && ! vals[0][1] && ! vals[1][0] &&   vals[1][1])
//         printf("\u259E");
//       if (! vals[0][0] && ! vals[0][1] && ! vals[1][0] && ! vals[1][1])
//         printf(" ");
     }
     printf("\n");
   }

}

void testpoint(/*!!!*/struct d_complex c/*!!!*/, int *dest){

// Does the iteration z=z*z+c, until |z| > 2 when point is known to be outside set
// If loop count reaches MAXITER, point is considered to be inside the set

       struct d_complex z;
       int iter;
       double temp;

       z=c;
       for (iter=0; iter<MAXITER; iter++){
         temp = (z.r*z.r)-(z.i*z.i)+c.r;
         z.i = z.r*z.i*2+c.i;
         z.r = temp;
         if ((z.r*z.r+z.i*z.i)>4.0) {
           #pragma omp atomic
           numoutside++;
           *dest = 1;
           break;
         }
       }
}

